# See full reference at https://devenv.sh/reference/options/
{ pkgs, inputs, phps, lib, config, modulesPath,... }:

{

  env.APP_NAME = "bob";

  # https://devenv.sh/packages/
  packages = with pkgs; [
    inputs.version.defaultPackage."${builtins.currentSystem}"
    git
    gcc12
    go-task
    blackbox
    blackbox-terminal
    jq
    delve
    gdlv
    wget
    glab
    unixtools.xxd
    libffi
    zlib
    procps
    php81Extensions.xdebug
    ranger
    meld
    gnused
    coreutils-full
    gnugrep
    gnumake
    util-linux
    httpie
    netcat
    memcached
    fd    
  ];


  # https://devenv.sh/languages/
  # languages.nix.enable = true;
  languages = {
    go = { enable = true; };
  };
  
   difftastic.enable = true;


  scripts.calc-nix-hash.exec = ''
  
cat <<'EOF' > ${config.devenv.root}/hashes.nix
{
  hashValue = "sha256-AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=";
}
EOF

HASH=$(nix build 2>&1 | grep 'got:'  | awk '{print $2}')

cat <<EOF > ${config.devenv.root}/hashes.nix
{
    hashValue = "''${HASH}";
}
EOF

nix build

  
  '';
    
  # This script is executed when the app is built
  # You can use it to build the app  
  scripts.build-app.exec =  ''
#!${pkgs.bash}/bin/bash
#set -euo pipefail
set -x

PATH="''${PATH}":${pkgs.coreutils}/bin
PATH="''${PATH}":${pkgs.findutils}/bin
PATH="''${PATH}":${pkgs.jq}/bin/
PATH="''${PATH}":${pkgs.rsync}/bin/
PATH="''${PATH}":${pkgs.bash}/bin/
PATH="''${PATH}":${pkgs.curl}/bin/
PATH="''${PATH}":${pkgs.moreutils}/bin/
PATH="''${PATH}":${pkgs.gnutar}/bin
PATH="''${PATH}":${pkgs.gzip}/bin/
PATH="''${PATH}":${pkgs.procps}/bin/
PATH="''${PATH}":${pkgs.exa}/bin/
PATH="''${PATH}":${pkgs.git}/bin/
PATH="''${PATH}":${pkgs.gnugrep}/bin/
PATH="''${PATH}":${inputs.version.defaultPackage."${builtins.currentSystem}"}/bin/

export PATH


if [ -z "'CI_JOB_TOKEN" ]; then
    version=''$(git describe --tags --always --dirty)
    buildDate=''$(date -u +"%Y-%m-%dT%H:%M:%SZ")
else
    version=''$(version predict)
    buildDate=''$(version date)
fi

if [[ -z "''${HOME}" ]]; then
    HOME=$(pwd)
fi

${config.devenv.root}/bin/go-compile.sh \
                    "${config.devenv.root}/source/" \
                    "${config.devenv.root}/dist"/ \
                    "bob" \
                    "main" \
                    "''${version}" \
                    "''${buildDate}"


'';

  # This scritp is used to deploy the app to the gitlab registry
  # It is used by the gitlab-ci.yml file
  # The environment variables are set in the gitlab project settings
  scripts.deploy-app.exec =  ''
#!${pkgs.bash}/bin/bash

PATH="''${PATH}":${pkgs.coreutils}/bin
PATH="''${PATH}":${pkgs.jq}/bin/
PATH="''${PATH}":${pkgs.curl}/bin/
PATH="''${PATH}":${pkgs.moreutils}/bin/
PATH="''${PATH}":${pkgs.gnutar}/bin
PATH="''${PATH}":${pkgs.gzip}/bin/
PATH="''${PATH}":${pkgs.exa}/bin/
PATH="''${PATH}":${pkgs.git}/bin/
PATH="''${PATH}":${inputs.version.defaultPackage."${builtins.currentSystem}"}/bin/

export -f PATH

if [[ -f .env-gitlab-ci ]]; then
    source .env-gitlab-ci
    rm .env-gitlab-ci
fi

set -x
## if $HOME not set, set it to current directory
if [[ -z "''${HOME}" ]]; then
    HOME=$(pwd)
fi

export HOME

git config user.email "''${GITLAB_USER_EMAIL}"
git config user.name "''${GITLAB_USER_NAME:-"Gitlab CI"}"
git config pull.rebase true
git config http.sslVerify "false"
git remote set-url origin https://pad:''${GITLAB_TOKEN}@''${CI_REPOSITORY_URL#*@}

git fetch --all --tags --unshallow
git reset --hard origin/master
git checkout $CI_COMMIT_REF_NAME
git pull origin $CI_COMMIT_REF_NAME


if [ ! -z "''${CI_PROJECT_DIR}" ]; then
    echo "CI_PROJECT_DIR is set, using it as project root."
    project_root=$(realpath "''${CI_PROJECT_DIR}")/
elif [ ! -z "''${DEVENV_ROOT}" ]; then
    echo "DEVENV_ROOT is set, using it as project root."
    project_root=$(realpath "''${DEVENV_ROOT}")/
else
    echo "Error: DEVENV_ROOT or CI_PROJECT_DIR environment variables are not set."
    exit 1  
fi

if [ ! -d "''${project_root}" ]; then
    echo "Error: Project root directory does not seem to be valid."
    echo "Check the DEVENV_ROOT or CI_PROJECT_DIR environment variables."
    exit 1
fi

if [ -z "'CI_JOB_TOKEN" ]; then
    echo "Error: CI_JOB_TOKEN variable is not set."
    exit 1
fi

git --no-pager log --decorate=short --pretty=oneline

## the version should be the same as in the build task
if ! version auto --git --verbose
then
    echo "ERROR: Could not update version."
    exit 1
fi 

git --no-pager log --decorate=short --pretty=oneline

gitVersion=$(version print --git)


for file in $(exa "''${project_root}"dist/*); do
    filname=$(basename ''${file})
    curl -v --insecure -H "JOB-TOKEN: ''${CI_JOB_TOKEN}" --upload-file \
        ''${file} \
        https://gitlab.schukai.com/api/v4/projects/''${CI_PROJECT_ID}/packages/generic/${config.env.APP_NAME}/''${gitVersion}/''${filname}  
done

git push -o ci.skip origin ''${CI_COMMIT_REF_NAME} --tags

echo "done"

  '';


}
